describe Verifier, "#verify" do

  # subject is an instance of class, that includes Verifier and has #foo method
  before  { Test = Class.new                          }
  before  { Test.include described_class              }
  before  { Test.send(:define_method, :foo) { "foo" } }
  subject { Test.new                                  }
  after   { Object.send :remove_const, :Test          }

  it "is defined" do
    expect(subject).to respond_to(:verify).with(2).arguments
  end

  context "when a block isn't given" do

    it "raises SyntaxError" do
      expect { subject.verify(:foo, :valid) }.to raise_error SyntaxError
    end
  end

  context "when a method is not defined" do

    it "raises Verifier::MethodNotDefined" do
      expect { subject.verify(:bar, :valid) { true } }
        .to raise_error Verifier::MethodNotDefined
    end

    it "displays proper error message" do
      expected_message = "Method not defined: Test#bar"

      begin
        subject.verify(:bar, :valid) { true }
      rescue => error
        expect(error.message).to eq expected_message
      end
    end
  end

  context "when a method raises RuntimeError" do

    before { Test.send(:define_method, :bar) { fail } }

    it "raises Verifier::MethodFails" do
      expect { subject.verify(:bar, :valid) { true } }
        .to raise_error Verifier::MethodFails
    end

    it "displays proper error message" do
      expected_message = "Method fails: Test#bar"

      begin
        subject.verify(:bar, :valid) { true }
      rescue => error
        expect(error.message).to eq expected_message
      end
    end
  end

  context "when a block raises RuntimeError" do

    it "raises Verifier::Invalid" do
      expect { subject.verify(:foo, :valid) { fail RuntimeError } }
        .to raise_error Verifier::Invalid
    end

    it "displays proper error message" do
      expected_message = "Test#foo verification failed: \"foo\" not valid"

      begin
        subject.verify(:foo, :valid) { false }
      rescue => error
        expect(error.message).to eq expected_message
      end
    end
  end

  context "when a block returns false" do

    it "raises Verifier::Invalid" do
      expect { subject.verify(:foo, :valid) { false } }
        .to raise_error Verifier::Invalid
    end

    it "displays proper error message" do
      expected_message = "Test#foo verification failed: \"foo\" not valid"

      begin
        subject.verify(:foo, :valid) { false }
      rescue => error
        expect(error.message).to eq expected_message
      end
    end
  end

  context "when a block returns nil" do

    it "raises Verifier::Invalid" do
      expect { subject.verify(:foo, :valid) { nil } }
        .to raise_error Verifier::Invalid
    end
  end

  context "when a block returns true" do

    it "doesn't raise error" do
      expect { subject.verify(:foo, :valid) { true } }.not_to raise_error
    end
  end

  context "when negation of a block is truthy" do

    let(:null) { double :null, :! => true }

    it "raises Verifier::Invalid" do
      expect { subject.verify(:foo, :valid) { null } }
        .to raise_error Verifier::Invalid
    end
  end

  context "when a block returns false in a variable's scope" do

    it "raises Verifier::Invalid" do
      expect { subject.verify(:foo, :valid) { self != "foo" } }
        .to raise_error Verifier::Invalid
    end
  end

  context "when a block returns true in a variable's scope" do

    it "doesn't raise error" do
      expect { subject.verify(:foo, :valid) { self == "foo" } }
        .not_to raise_error
    end
  end
end
